import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:lori_sdk/data/lori_api.dart';
import 'package:lori_sdk/data/models/user_promotions/userPromotion.dart';

class SalesMarketingApi {
  static const LIST_USER_PROMOTIONS = "sales/marketing/buyers/promotions";
  static const ACTIVATE_USER_PROMOTION = "sales/marketing/buyers/promotions/active";

  Dio _requester;

  SalesMarketingApi({@required Dio requester}) : this._requester = requester;

  ///
  /// region User Promotions part. All methods related to User Promotions module.
  /// [START]
  ///

  /// Load all users promotions for the current user.
  Future<ApiCallback<List<UserPromotion>>> loadUserPromotions() async {
    try {
      var response = await _requester.get(
          LIST_USER_PROMOTIONS,
          options: Options(responseType: ResponseType.json)
      );
      var responseData = response.data as List;
      var promotions = List<UserPromotion>();
      for (var row in responseData) {
        promotions.add(UserPromotion.fromJson(row));
      }
      // Callback invocation
      return ApiCallback<List<UserPromotion>>(isSuccessful: true, resultData: promotions);
    } on DioError catch (e) {
      return ApiCallback<List<UserPromotion>>().defaultErrorCallback(e);
    }
  }

  /// Activate a promotion for the current user give it's id.
  Future<ApiCallback<void>> activateUserPromotion(String externalId) async {
    try {
      Map<String, String> body = {
        "partnerPromotionId": externalId,
      };
      await _requester.post(
          ACTIVATE_USER_PROMOTION,
          queryParameters: body,
          options: Options(responseType: ResponseType.json));
      // Callback invocation
      return ApiCallback<void>(isSuccessful: true);
    } on DioError catch (e) {
      return ApiCallback<void>().defaultErrorCallback(e);
    }
  }

  ///
  /// User Promotions part. All methods related to User Promotions module.
  /// endregion [START]
  ///
}