import 'dart:async';

import 'package:lori_sdk/data/database/interfaces/authResponseDao.dart';
import 'package:lori_sdk/data/models/authResponse.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';

class MobileAuthResponseDao implements AuthResponseDao {
  Database db;

  Future open() async {
    if (db == null || !db.isOpen) {
      var databasesPath = await getDatabasesPath();
      String path = join(databasesPath, 'lori.db');
      db = await openDatabase(path, version: 1,
          onCreate: (Database db, int version) async {
        await db.execute('''
        create table $tableName (
          $columnToken text not null,
          $columnRefreshToken text,
          $columnExpiresIn integer,
          $columnTokenType text not null,
          $columnResponseType integer,
          $columnResponseCreateDate integer
        )
      ''');
      });
    }
  }

  Future<void> insertOrUpdate(AuthResponse response) async {
    var storedResponse = await get(response.responseType);
    if (storedResponse == null) {
      return insert(response);
    } else {
      return update(response);
    }
  }

  Future<void> insert(AuthResponse response) async {
    await open();
    if (response.refreshToken == null) response.refreshToken = '';
    response.responseCreatedDate = DateTime.now().millisecondsSinceEpoch;
    await db.insert(tableName, response.toMap());
    return response;
  }

  Future<void> update(AuthResponse response) async {
    await open();
    if (response.refreshToken == null) response.refreshToken = '';
    response.responseCreatedDate = DateTime.now().millisecondsSinceEpoch;
    return db.update(tableName, response.toMap(),
        where: '$columnResponseType = ?', whereArgs: [response.responseType]);
  }

  Future<AuthResponse> get(int responseType) async {
    await open();
    List<Map> maps = await db.query(tableName,
        where: '$columnResponseType = ?', whereArgs: [responseType]);
    if (maps != null && maps.length > 0) {
      var response = AuthResponse('', '', 0, '');
      response.fromMap(maps.first);
      return response;
    }
    return null;
  }

  Future<int> delete(int responseType) async {
    await open();
    return db.delete(tableName,
        where: '$columnResponseType = ?', whereArgs: [responseType]);
  }

  Future<void> deleteAll() async {
    await open();
    return db.delete(
      tableName,
    );
  }

  Future close() async => db.close();
}

AuthResponseDao getAuthResponseDao() => MobileAuthResponseDao();
